var SocketIORemoteEndpoint,
  slice = [].slice;

SocketIORemoteEndpoint = (function() {
  function SocketIORemoteEndpoint() {}

  SocketIORemoteEndpoint.prototype.initialize = function(options, callback) {
    if (options == null) {
      options = {};
    }
    if (callback == null) {
      callback = function() {};
    }
    this._processOptions(options);
    this._io.sockets.on('connection', (function(_this) {
      return function(socket) {
        socket.on('RPC_Request', function(rpcRequest) {
          return _this._handleRpcRequestEvent(rpcRequest, socket);
        });
        socket.on('JoinRoom', function(roomName) {
          return _this._rpcRequestMiddleware(roomName, socket).then(function() {
            return socket.join(roomName);
          })["catch"](function(error) {});
        });
        return socket.on('LeaveRoom', function(roomName) {
          return socket.leave(roomName);
        });
      };
    })(this));
    return callback();
  };

  SocketIORemoteEndpoint.prototype.setRPCHandler = function(_handleRPCRequest) {
    this._handleRPCRequest = _handleRPCRequest;
  };

  SocketIORemoteEndpoint.prototype._processOptions = function(options) {
    if (options.ioInstance) {
      this._io = options.ioInstance;
    } else {
      this._io = require('socket.io')();
      this._io.listen(3000);
    }
    if (options.rpcRequestMiddleware) {
      return this._rpcRequestMiddleware = options.rpcRequestMiddleware;
    } else {
      return this._rpcRequestMiddleware = function() {
        return {
          then: function(callback) {
            callback();
            return {
              "catch": function() {}
            };
          }
        };
      };
    }
  };

  SocketIORemoteEndpoint.prototype._handleRpcRequestEvent = function(rpcRequest, socket) {
    var emitRpcResponse;
    emitRpcResponse = (function(_this) {
      return function(error, response) {
        var rpcId;
        rpcId = rpcRequest.rpcId;
        return socket.emit('RPC_Response', {
          rpcId: rpcId,
          err: error,
          data: response
        });
      };
    })(this);
    return this._rpcRequestMiddleware(rpcRequest, socket).then((function(_this) {
      return function() {
        return _this._handleRPCRequest(rpcRequest, emitRpcResponse);
      };
    })(this))["catch"](function(error) {
      return emitRpcResponse(error, null);
    });
  };

  SocketIORemoteEndpoint.prototype.publish = function() {
    var aggregateId, arg, context, domainEventName, fullEventName, i, payload;
    context = arguments[0], arg = 3 <= arguments.length ? slice.call(arguments, 1, i = arguments.length - 1) : (i = 1, []), payload = arguments[i++];
    domainEventName = arg[0], aggregateId = arg[1];
    fullEventName = this._getFullEventName(context, domainEventName, aggregateId);
    return this._io.to(fullEventName).emit(fullEventName, payload);
  };

  SocketIORemoteEndpoint.prototype._getFullEventName = function(context, domainEventName, aggregateId) {
    var fullEventName;
    fullEventName = context;
    if (domainEventName) {
      fullEventName += "/" + domainEventName;
    }
    if (aggregateId) {
      fullEventName += "/" + aggregateId;
    }
    return fullEventName;
  };

  SocketIORemoteEndpoint.prototype.close = function() {
    return this._io.close();
  };

  return SocketIORemoteEndpoint;

})();

module.exports = new SocketIORemoteEndpoint;
